/* Automatically generated nanopb constant definitions */
/* Generated by nanopb-0.4.6-dev */

#include "flipper.pb.h"
#if PB_PROTO_HEADER_VERSION != 40
#error Regenerate this file with the current version of nanopb generator.
#endif

PB_BIND(PB_Empty, PB_Empty, AUTO)


PB_BIND(PB_StopSession, PB_StopSession, AUTO)


PB_BIND(PB_Main, PB_Main, 2)


PB_BIND(PB_Region, PB_Region, AUTO)


PB_BIND(PB_Region_Band, PB_Region_Band, AUTO)




