/* Automatically generated nanopb constant definitions */
/* Generated by nanopb-0.4.6-dev */

#include "gpio.pb.h"
#if PB_PROTO_HEADER_VERSION != 40
#error Regenerate this file with the current version of nanopb generator.
#endif

PB_BIND(PB_Gpio_SetPinMode, PB_Gpio_SetPinMode, AUTO)


PB_BIND(PB_Gpio_SetInputPull, PB_Gpio_SetInputPull, AUTO)


PB_BIND(PB_Gpio_GetPinMode, PB_Gpio_GetPinMode, AUTO)


PB_BIND(PB_Gpio_GetPinModeResponse, PB_Gpio_GetPinModeResponse, AUTO)


PB_BIND(PB_Gpio_ReadPin, PB_Gpio_ReadPin, AUTO)


PB_BIND(PB_Gpio_ReadPinResponse, PB_Gpio_ReadPinResponse, AUTO)


PB_BIND(PB_Gpio_WritePin, PB_Gpio_WritePin, AUTO)






