/* Automatically generated nanopb constant definitions */
/* Generated by nanopb-0.4.6-dev */

#include "storage.pb.h"
#if PB_PROTO_HEADER_VERSION != 40
#error Regenerate this file with the current version of nanopb generator.
#endif

PB_BIND(PB_Storage_File, PB_Storage_File, AUTO)


PB_BIND(PB_Storage_InfoRequest, PB_Storage_InfoRequest, AUTO)


PB_BIND(PB_Storage_InfoResponse, PB_Storage_InfoResponse, AUTO)


PB_BIND(PB_Storage_TimestampRequest, PB_Storage_TimestampRequest, AUTO)


PB_BIND(PB_Storage_TimestampResponse, PB_Storage_TimestampResponse, AUTO)


PB_BIND(PB_Storage_StatRequest, PB_Storage_StatRequest, AUTO)


PB_BIND(PB_Storage_StatResponse, PB_Storage_StatResponse, AUTO)


PB_BIND(PB_Storage_ListRequest, PB_Storage_ListRequest, AUTO)


PB_BIND(PB_Storage_ListResponse, PB_Storage_ListResponse, AUTO)


PB_BIND(PB_Storage_ReadRequest, PB_Storage_ReadRequest, AUTO)


PB_BIND(PB_Storage_ReadResponse, PB_Storage_ReadResponse, AUTO)


PB_BIND(PB_Storage_WriteRequest, PB_Storage_WriteRequest, AUTO)


PB_BIND(PB_Storage_DeleteRequest, PB_Storage_DeleteRequest, AUTO)


PB_BIND(PB_Storage_MkdirRequest, PB_Storage_MkdirRequest, AUTO)


PB_BIND(PB_Storage_Md5sumRequest, PB_Storage_Md5sumRequest, AUTO)


PB_BIND(PB_Storage_Md5sumResponse, PB_Storage_Md5sumResponse, AUTO)


PB_BIND(PB_Storage_RenameRequest, PB_Storage_RenameRequest, AUTO)


PB_BIND(PB_Storage_BackupCreateRequest, PB_Storage_BackupCreateRequest, AUTO)


PB_BIND(PB_Storage_BackupRestoreRequest, PB_Storage_BackupRestoreRequest, AUTO)




