/* Automatically generated nanopb constant definitions */
/* Generated by nanopb-0.4.6-dev */

#include "status.pb.h"
#if PB_PROTO_HEADER_VERSION != 40
#error Regenerate this file with the current version of nanopb generator.
#endif

PB_BIND(PB_Status_PingRequest, PB_Status_PingRequest, AUTO)


PB_BIND(PB_Status_PingResponse, PB_Status_PingResponse, AUTO)



