/* Automatically generated nanopb constant definitions */
/* Generated by nanopb-0.4.6-dev */

#include "property.pb.h"
#if PB_PROTO_HEADER_VERSION != 40
#error Regenerate this file with the current version of nanopb generator.
#endif

PB_BIND(PB_Property_GetRequest, PB_Property_GetRequest, AUTO)


PB_BIND(PB_Property_GetResponse, PB_Property_GetResponse, AUTO)



