/* Automatically generated nanopb constant definitions */
/* Generated by nanopb-0.4.6-dev */

#include "system.pb.h"
#if PB_PROTO_HEADER_VERSION != 40
#error Regenerate this file with the current version of nanopb generator.
#endif

PB_BIND(PB_System_PingRequest, PB_System_PingRequest, AUTO)


PB_BIND(PB_System_PingResponse, PB_System_PingResponse, AUTO)


PB_BIND(PB_System_RebootRequest, PB_System_RebootRequest, AUTO)


PB_BIND(PB_System_DeviceInfoRequest, PB_System_DeviceInfoRequest, AUTO)


PB_BIND(PB_System_DeviceInfoResponse, PB_System_DeviceInfoResponse, AUTO)


PB_BIND(PB_System_FactoryResetRequest, PB_System_FactoryResetRequest, AUTO)


PB_BIND(PB_System_GetDateTimeRequest, PB_System_GetDateTimeRequest, AUTO)


PB_BIND(PB_System_GetDateTimeResponse, PB_System_GetDateTimeResponse, AUTO)


PB_BIND(PB_System_SetDateTimeRequest, PB_System_SetDateTimeRequest, AUTO)


PB_BIND(PB_System_DateTime, PB_System_DateTime, AUTO)


PB_BIND(PB_System_PlayAudiovisualAlertRequest, PB_System_PlayAudiovisualAlertRequest, AUTO)


PB_BIND(PB_System_ProtobufVersionRequest, PB_System_ProtobufVersionRequest, AUTO)


PB_BIND(PB_System_ProtobufVersionResponse, PB_System_ProtobufVersionResponse, AUTO)


PB_BIND(PB_System_UpdateRequest, PB_System_UpdateRequest, AUTO)


PB_BIND(PB_System_UpdateResponse, PB_System_UpdateResponse, AUTO)


PB_BIND(PB_System_PowerInfoRequest, PB_System_PowerInfoRequest, AUTO)


PB_BIND(PB_System_PowerInfoResponse, PB_System_PowerInfoResponse, AUTO)





