/* Automatically generated nanopb constant definitions */
/* Generated by nanopb-0.4.6-dev */

#include "application.pb.h"
#if PB_PROTO_HEADER_VERSION != 40
#error Regenerate this file with the current version of nanopb generator.
#endif

PB_BIND(PB_App_StartRequest, PB_App_StartRequest, AUTO)


PB_BIND(PB_App_LockStatusRequest, PB_App_LockStatusRequest, AUTO)


PB_BIND(PB_App_LockStatusResponse, PB_App_LockStatusResponse, AUTO)


PB_BIND(PB_App_AppExitRequest, PB_App_AppExitRequest, AUTO)


PB_BIND(PB_App_AppLoadFileRequest, PB_App_AppLoadFileRequest, 2)


PB_BIND(PB_App_AppButtonPressRequest, PB_App_AppButtonPressRequest, 2)


PB_BIND(PB_App_AppButtonReleaseRequest, PB_App_AppButtonReleaseRequest, AUTO)


PB_BIND(PB_App_AppStateResponse, PB_App_AppStateResponse, AUTO)


PB_BIND(PB_App_GetErrorRequest, PB_App_GetErrorRequest, AUTO)


PB_BIND(PB_App_GetErrorResponse, PB_App_GetErrorResponse, AUTO)


PB_BIND(PB_App_DataExchangeRequest, PB_App_DataExchangeRequest, AUTO)




